@@ -14,8 +14,12 @@
 #' matrix.
 #' 
 #' Available types currently include...
-#' @slot dfm a \link{dfm-class} document-feature matrix
+#' @slot x  the dfm on which the wordscores model was called
+#' @slot y  supervised labels (for supervised methods only)
+#' @slot call  the function call that fitted the model
+#' @slot method the type of textmodel, e.g. wordfish, wordscores, NB-
 #' @name textmodel_fitted-class
+#' @keywords internal textmodel
 #' @export
 setClass("textmodel_fitted",
          slots = c(x = "dfm", y = "ANY", call = "call", method = "character"))
@@ -65,7 +69,7 @@ setClass("textmodel_fitted",
 #'   is not terribly obvious. (Blame it on the S3 system.)
 #' @export
 #' @examples
-#' ieDfm <- dfm(ie2010Corpus, verbose=FALSE)
+#' ieDfm <- dfm(data_corpus_irishbudget2010, verbose=FALSE)
 #' refscores <- c(rep(NA, 4), -1, 1, rep(NA, 8))
 #' ws <- textmodel(ieDfm, refscores, model="wordscores", smooth=1)
 #'
@@ -145,7 +149,7 @@ setMethod("textmodel", signature(x = "formula", y="missing", data="dfm", model =
                   y <- stats::model.response(mf)
                   # catm("HERE\n")
                   textmodel(data[which(docnames(data) %in% names(y)), 
-                                 which(features(data) %in% names(mf))], 
+                                 which(featnames(data) %in% names(mf))], 
                             y, model = model, ...)
               })
 
