@@ -5,8 +5,14 @@
 #' @param x tokens object to which dictionary or thesaurus will be supplied
 #' @param dictionary the \link{dictionary}-class object that will be applied to 
 #'   \code{x}
+#' @param levels integers specifying the levels of entries in a hierarchical
+#'   dictionary that will be applied.  The top level is 1, and subsequent levels
+#'   describe lower nesting levels.  Values may be combined, even if these
+#'   levels are not contiguous, e.g. `levels = c(1:3)` will collapse the second
+#'   level into the first, but record the third level (if present) collapsed below
+#'   the first.  (See examples.)
 #' @inheritParams valuetype
-#' @param concatenator a charactor that connect words in multi-words entries
+#' @param concatenator a charactor that connect words in multi-words entries in \code{x}
 #' @param case_insensitive ignore the case of dictionary values if \code{TRUE} 
 #'   uppercase to distinguish them from other features
 #' @param capkeys if TRUE, convert dictionary keys to uppercase to distinguish 
@@ -14,32 +20,53 @@
 #' @param exclusive if \code{TRUE}, remove all features not in dictionary, 
 #'   otherwise, replace values in dictionary with keys while leaving other 
 #'   features unaffected
+#' @param multiword if \code{FALSE}, multi-word entries in dictionary are treated
+#'   as single tokens
 #' @param verbose print status messages if \code{TRUE}
 #' @examples
 #' toks <- tokens(data_corpus_inaugural)
 #' dict <- dictionary(list(country = "united states", 
 #'                    law=c('law*', 'constitution'), 
 #'                    freedom=c('free*', 'libert*')))
-#' dfm(tokens_lookup(toks, dict, 'glob', verbose = TRUE))
+#' dfm(tokens_lookup(toks, dict, valuetype='glob', verbose = TRUE))
 #' 
 #' dict_fix <- dictionary(list(country = "united states", 
 #'                        law = c('law', 'constitution'), 
 #'                        freedom = c('freedom', 'liberty'))) 
 #' dfm(applyDictionary(toks, dict_fix, valuetype='fixed'))
 #' dfm(tokens_lookup(toks, dict_fix, valuetype='fixed'))
+#' 
+#' # hierarchical dictionary example
+#' txt <- c(d1 = "The United States has the Atlantic Ocean and the Pacific Ocean.",
+#'          d2 = "Britain and Ireland have the Irish Sea and the English Channel.")
+#' toks <- tokens(txt)
+#' dict <- dictionary(list(US = list(Countries = c("States"), 
+#'                                   oceans = c("Atlantic", "Pacific")),
+#'                         Europe = list(Countries = c("Britain", "Ireland"),
+#'                                       oceans = list(west = "Irish Sea", 
+#'                                                     east = "English Channel"))))
+#' tokens_lookup(toks, dict, levels = 1)
+#' tokens_lookup(toks, dict, levels = 2)
+#' tokens_lookup(toks, dict, levels = 1:2)
+#' tokens_lookup(toks, dict, levels = 3)
+#' tokens_lookup(toks, dict, levels = c(1,3))
+#' tokens_lookup(toks, dict, levels = c(2,3))
 #' @importFrom RcppParallel RcppParallelLibs
 #' @export
-tokens_lookup <- function(x, dictionary,
+tokens_lookup <- function(x, dictionary, levels = 1:5,
                           valuetype = c("glob", "regex", "fixed"), 
+                          concatenator = ' ',
                           case_insensitive = TRUE,
                           capkeys = FALSE,
-                          concatenator = " ",
                           exclusive = TRUE,
+                          multiword = TRUE,
                           verbose = FALSE) {
     
-    valuetype <- match.arg(valuetype)
     names_org <- names(x)
     attrs_org <- attributes(x)
+    concatenator_dict <- attr(dictionary, 'concatenator')
+    dictionary <- dictionary_flatten(dictionary, levels)
+    valuetype <- match.arg(valuetype)
     
     # Generate all combinations of type IDs
     entries_id <- list()
@@ -48,9 +75,21 @@ tokens_lookup <- function(x, dictionary,
     
     index <- index_regex(types, valuetype, case_insensitive) # index types before the loop
     if (verbose) 
-        message('Registering ', length(unlist(dictionary)), ' entiries in the dictionary...');
+        message('Registering ', length(unlist(dictionary)), ' entries in the dictionary...');
     for (h in 1:length(dictionary)) {
-        entries <- stringi::stri_split_fixed(dictionary[[h]], concatenator)
+        entries <- dictionary[[h]]
+        
+        # Substitute dictionary's concatenator with tokens' concatenator 
+        if (concatenator != concatenator_dict)
+            entries <- stringi::stri_replace_all_fixed(entries, concatenator_dict, concatenator)
+        
+        # Separate entries by concatenator
+        if (multiword) {
+            entries <- stringi::stri_split_fixed(entries, concatenator)
+        } else {
+            entries <- as.list(entries)
+        } 
+
         entries_temp <- regex2id(entries, types, valuetype, case_insensitive, index)
         entries_id <- c(entries_id, entries_temp)
         keys_id <- c(keys_id, rep(h, length(entries_temp)))
@@ -81,6 +120,5 @@ tokens_lookup <- function(x, dictionary,
     names(x) <- names_org
     attr(x, "what") <- "dictionary"
     attr(x, "dictionary") <- dictionary
-    
-    return(x)
+    tokens_hashed_recompile(x)
 }
