@@ -10,25 +10,28 @@ using namespace ngrams;
 
 
 Text replace(Text tokens, 
-             std::size_t span_max,
-             MapNgrams &map_words){
+             const std::vector<std::size_t> &spans,
+             const MapNgrams &map_words){
     
-    if(tokens.size() == 0) return {}; // return empty vector for empty text
+    if (tokens.size() == 0) return {}; // return empty vector for empty text
     
     unsigned int filler = std::numeric_limits<unsigned int>::max(); // use largest limit as filler
     bool match = false;
-    for (std::size_t span = span_max; span > 0; span--){ // substitution starts from the longest sequences
+    for (std::size_t span : spans) { // substitution starts from the longest sequences
+        if (tokens.size() < span) continue;
         for (std::size_t i = 0; i < tokens.size() - (span - 1); i++){
             Ngram ngram(tokens.begin() + i, tokens.begin() + i + span);
-            unsigned int &id = map_words[ngram];
-            if(id){
+            auto it = map_words.find(ngram);
+            if (it != map_words.end()) {
+            //unsigned int id = map_words[ngram];
+            //if (id) {
                 match = true;
                 std::fill(tokens.begin() + i + 1, tokens.begin() + i + span, filler); // fill subsequent tokens
-                tokens[i] = id;
+                tokens[i] = it->second;
             }
         }
     }
-    if(match) tokens.erase(std::remove(tokens.begin(), tokens.end(), filler), tokens.end());
+    if (match) tokens.erase(std::remove(tokens.begin(), tokens.end(), filler), tokens.end());
     return tokens;
 }
 
@@ -36,18 +39,18 @@ struct replace_mt : public Worker{
     
     Texts &input;
     Texts &output;
-    std::size_t span_max;
-    MapNgrams &map_words;
+    const std::vector<std::size_t> &spans;
+    const MapNgrams &map_words;
     
     // Constructor
-    replace_mt(Texts &input_, Texts &output_, std::size_t span_max_, MapNgrams &map_words_):
-              input(input_), output(output_), span_max(span_max_), map_words(map_words_) {}
+    replace_mt(Texts &input_, Texts &output_, const std::vector<std::size_t> &spans_, MapNgrams &map_words_):
+              input(input_), output(output_), spans(spans_), map_words(map_words_) {}
     
     // parallelFor calles this function with std::size_t
     void operator()(std::size_t begin, std::size_t end){
         //Rcout << "Range " << begin << " " << end << "\n";
-        for (std::size_t h = begin; h < end; h++){
-            output[h] = replace(input[h], span_max, map_words);
+        for (std::size_t h = begin; h < end; h++) {
+            output[h] = replace(input[h], spans, map_words);
         }
     }
 };
@@ -64,32 +67,35 @@ struct replace_mt : public Worker{
  */
 
 // [[Rcpp::export]]
-List qatd_cpp_tokens_replace(List texts_, 
-                             List words_,
-                             IntegerVector ids_){
+List qatd_cpp_tokens_replace(const List &texts_, 
+                             const List &words_,
+                             const IntegerVector &ids_){
     
     Texts input = Rcpp::as<Texts>(texts_);
-    List words = words_;
-    IntegerVector ids = ids_;
+    const List words = words_;
+    const IntegerVector ids = ids_;
 
     MapNgrams map_words;
-    std::size_t span_max = 0;
-    for (unsigned int g = 0; g < words.size(); g++){
-        if(has_na(words[g])) continue;
+    std::vector<std::size_t> spans(words.size());
+    for (unsigned int g = 0; g < words.size(); g++) {
+        if (has_na(words[g])) continue;
         Ngram word = words[g];
         map_words[word] = ids[g];
-        if(span_max < word.size()) span_max = word.size();
+        spans[g] = word.size();
     }
+    sort(spans.begin(), spans.end());
+    spans.erase(unique(spans.begin(), spans.end()), spans.end());
+    std::reverse(std::begin(spans), std::end(spans));
     
     // dev::Timer timer;
     Texts output(input.size());
     // dev::start_timer("Token replace", timer);
     #if RCPP_PARALLEL_USE_TBB
-    replace_mt replace_mt(input, output, span_max, map_words);
+    replace_mt replace_mt(input, output, spans, map_words);
     parallelFor(0, input.size(), replace_mt);
     #else
-    for (std::size_t h = 0; h < input.size(); h++){
-        output[h] = replace(input[h], span_max, map_words);
+    for (std::size_t h = 0; h < input.size(); h++) {
+        output[h] = replace(input[h], spans, map_words);
     }
     #endif
     // dev::stop_timer("Token replace", timer);
